% GamutFit(x,y,bn)
%
% Use support vector regression to fit the deviation of out of
% gamut points---- finds f such that f(x) = y.
%
% -- Ayan Chakrabarti <ayanc@eecs.harvard.edu>
function GamutFit(x,y,bn)

% Select validation set
rand('seed',0); idx = randperm(size(x,2));
idx = idx(1:round(1/2*length(idx)));
xv = x(:,idx); yv = y(:,idx);
xt = x; xt(:,idx) = []; yt = y; yt(:,idx) = [];

gs = 2.^[2:0.5:7];
scrs = zeros(3,length(gs));

% Cross Validation
for i = 1:length(gs)
    fitg(xv,yv,bn,repmat(gs(i),[3 1]));
    yp = GamutEst(xt,bn);
    scrs(:,i) = sum((yp-yt).^2,2);
    fprintf('Scores for g=%f: [%f %f %f]\n',gs(i),scrs(:,i));
end;
g = zeros(3,1);
for i = 1:3
    idx = find(scrs(i,:) == min(scrs(i,:)));
    idx = idx(1);
    g(i) = gs(idx);
end;
fprintf('Selecting g = %f,%f,%f\n',g);
fitg(x,y,bn,g);

function fitg(x,y,bn,g)

global SVM_TRAIN;

C = 10^6 / size(x,2);

tm0 = [tempname '.svm'];

cmd = '';
for i = 1:3
  %%% Set SVM opts here
  opts = ['-t 2 -g ' sprintf('%f',g) ' -c ' sprintf('%f',C)  ' -h 0 '];

  tm = [tm0 num2str(i)];
  fid = fopen(tm,'wt');
  xyi = [y(i,:); x];
  fprintf(fid,'%f 1: %f 2: %f 3: %f\n',xyi);
  fclose(fid);
  
  outnm = sprintf('%s.c%d',bn,i);
    
  cmd = [cmd SVM_TRAIN ' -s 3 -p 1 ' opts ' ' tm ' ' outnm ' & '];
end;
system([cmd ' wait']);

delete([tm0 '1']);
delete([tm0 '2']);
delete([tm0 '3']);

  