% h = getHull(x,robustPer)
% 
% Compute the hull (in chromaticity space) to be applied
% later. robustPer (optional: default = 0) is the ratio of points we
% want to throw away to compute a smaller more "robust" hull.
%
% -- Ayan Chakrabarti <ayanc@eecs.harvard.edu>
function h = getHull(x,robustPer)

eps = 1e-3;
if nargin == 1
  robustPer = 0;
end;

idx = find(sum(x,1) > eps);
x = x(:,idx);

xr = x(1,:) ./ sum(x,1);
xg = x(2,:) ./ sum(x,1);

keep = (1-robustPer) * length(xr);
while 1
  k = convhull(xr,xg); k = k(1:end-1);
  if length(xr) - length(k) < keep
    break;
  end;
  xr(k) = []; xg(k) = [];
end;

h = []; h.eps = eps;
h.xr = xr(k); h.xg = xg(k);