% InvertBase(bn)
%
% Pre-process learned forward fits (saved with basename bn) to speed
% up numerical integration. Call this before calling InvertBaseQ.
%
% -- Ayan Chakrabarti <ayanc@eecs.harvard.edu>
function InvertBase(bn)

gridSize = 200;

stats = load('-mat',[bn '_stats.mat']);
Js = stats.rmse*2;


%[x,wts] = Uniform(gridSize);
[x,wts] = Poly2(gridSize);

idx = checkHull(x,stats.h);
x(:,idx) = []; wts(idx) = [];

y = Render(x,bn);

save('-mat',[bn '_INV.mat'],'x','y','wts','Js');


% Uniformly sample 
%%%%
function [x,wts] = Uniform(gridSize)

r = repmat(reshape(linspace(0,1,gridSize),[gridSize 1 1]),[1 gridSize gridSize]);
g = repmat(reshape(linspace(0,1,gridSize),[1 gridSize 1]),[gridSize 1 gridSize]);
b = repmat(reshape(linspace(0,1,gridSize),[1 1 gridSize]),[gridSize gridSize 1]);
r = r(:); g = g(:); b = b(:);

x = [r'; g'; b'];
wts = ones(1,length(r));

% Sample the square root uniformly
%%%%%%%%%
function [x,wts] = Poly2(gridSize)
[r,g,b] = meshgrid(linspace(0,1,gridSize));
x = [r(:).^2 g(:).^2 b(:).^2]';
wts = ( r(:) .* g(:) .* b(:) )';



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Old - not effective
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
function [x,wts] = Poly(gridSize,params)

coeffs = params.coeffs(1,:); deg = params.deg;
T = params.T;

% Learn inverse fit
xlim = linspace(0,max(T(:)),1000); 
jpg = coeffs*polyaug(xlim,deg); jpgMax = max(jpg(:)); 
invCoeffs = (polyaug(jpg,deg)'\xlim')';

% Sample in J
grid = linspace(-invCoeffs(1),jpgMax,gridSize);
grid = invCoeffs*polyaug(grid,deg);
[r,g,b] = meshgrid(grid);
r = r(:)'; g = g(:)'; b = b(:)';

x = inv(T)*[r;g;b];

dpoly = [1:deg].*invCoeffs(2:end);
wts = dpoly*polyaug(r,deg-1);
wts = wts .* (dpoly*polyaug(g,deg-1));
wts = wts .* (dpoly*polyaug(b,deg-1));
wts = abs(wts) / mean(abs(wts));

idx = find(min(x,[],1) >= 0 & max(x,[],1) <= 1);
x = x(:,idx);
wts = wts(idx);
