% idx = checkHull(x,h)
% 
% Find index of points that lie outside the convex chromaticity hull.
%
% -- Ayan Chakrabarti <ayanc@eecs.harvard.edu>
function idx = checkHull(x,h)

xr = x(1,:) ./ sum(x,1);
xg = x(2,:) ./ sum(x,1);

% Don't do anything to points whose luminance is < eps
idx = find(sum(x,1) > h.eps);

xr = xr(idx); xg = xg(idx);

% Find which points lie inside hull
inside = ones(1,length(xr));
for k = 1:length(h.xr)
  
  k1 = mod(k,length(h.xr))+1;
  k2 = mod(k1,length(h.xr))+1;
  
  orthV = null([h.xr(k) h.xg(k)] - [h.xr(k1) h.xg(k1)]);
  b = [h.xr(k) h.xg(k)]*orthV;
  
  sgn = sign([h.xr(k2) h.xg(k2)]*orthV - b);
  insk = sgn*(orthV'*[xr; xg] - b);
  inside = inside .* double(insk >= 0);
end;

% Outside points
idx2 = find(inside == 0);
fprintf('%d points outside hull of training data\n',length(idx2));
idx = idx(idx2);
