/* 
function wexpi = corrMax(r,g,b)

Assumes that lgwts and y are variables defined in the "caller" function.

Does the following:
  wexpi = -2*(y(1,:)*J(1,i)+y(2,:)*J(2,i)+y(3,:)*J(3,i));
  wexpi = lgwts-wexpi;
  wexpi = wexpi - max(wexpi);

Very little error checking, use only as directed!! 

-- Ayan Chakrabarti <ayanc@eecs.harvard.edu>
*/

#include "mex.h"

void corrMax(double r, double g, double b, 
	     double * y, double * lgw, 
	     double * w, mwSize sz) {

	double max;
	mwSize i;
	
	double *wi,tmp;

	wi = w;
	
	tmp = r*(*(y++)); tmp += g*(*(y++)); tmp += b*(*(y++));
	tmp = *(lgw++) + 2*tmp;
	*(w++) = tmp; max = tmp;
	for(i = 1; i < sz; i++) {
		tmp = r*(*(y++)); tmp += g*(*(y++)); tmp += b*(*(y++));
		tmp = *(lgw++) + 2*tmp;
		if(tmp > max)
			max = tmp;
		*(w++) = tmp;
	}
	w = wi;
	for(i = 0; i < sz; i++) {
		tmp = *w - max;
		*(w++) = tmp;
	}

}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	double r,g,b;
	mwSize sz, odims[2];
	const mxArray * yy, * ll;

	r = mxGetScalar(prhs[0]);
	g = mxGetScalar(prhs[1]);
	b = mxGetScalar(prhs[2]);

	yy = mexGetVariablePtr("caller","y");
	ll = mexGetVariablePtr("caller","lgwts");
	if(!yy || !ll) {
		return;
	}
	sz = mxGetNumberOfElements(ll);
	odims[0] = 1; odims[1] = sz;
        plhs[0] = mxCreateNumericArray(2,odims, mxDOUBLE_CLASS, mxREAL);
	corrMax(r,g,b,mxGetPr(yy),mxGetPr(ll),mxGetPr(plhs[0]),sz);
}

