% [mu,cv] = invJQ(J,bn)
%
% Compute the mean and covariance matrices of Raw given JPEG. Call
% with J as a 3xN matrix, returns mu as 3xN and cv as
% 3x3xN. Here bn is the basename with which the forward fits and
% InvertBase function were called.
%
% -- Ayan Chakrabarti <ayanc@eecs.harvard.edu>
function [mu cv] = invJQ(J,bn)

Q = 8; sQ = 16;
gridsize = 200;


d = load([bn '_INV.mat']);
y = d.y; x = d.x;
Js = d.Js;
lgwts = log(d.wts);
clear d;

mu = zeros(3,size(J,2));
cv = zeros(3,3,size(J,2));

y2 = sum(y.^2,1); 
Js22 = 2*Js^2;
lgwts = Js22*lgwts - y2;
thresh = -12*Js22;

lgFull = lgwts; yFull = y;

count = 1;

jrx = floor(J(1,:)/256*sQ);
for j = 0:(sQ-1)
  sqidx = find(jrx == j);
  if length(sqidx) == 0
    continue;
  end;
  sqidx = sqidx(:)';
  
  d = load('-mat',[bn '_INVQ_' num2str(j) '.mat']);
  for i = sqidx
    fprintf('\r Estimating %d of %d                ',count,size(J,2));
    count = count+1;
    
    jid = floor(J(:,i)/Q)+1; 
    jid(1) = mod(J(1,i),256/sQ/Q)+1;
    qidx = d.subIdx{jid(1),jid(2),jid(3)};
    
    lgwts = lgFull(qidx); y = yFull(:,qidx);
    wexpi = corrMax(J(1,i),J(2,i),J(3,i));
    
    idx = find(wexpi > thresh); 
    xi = x(:,qidx(idx));
    
    wts = exp(wexpi(idx)/Js22);
    
    xw = repmat(wts,[3 1]).*xi;
    m = sum(xw,2)/sum(wts);
    mu(:,i) = m;
    
    xw = (xi - repmat(m,[1 size(xi,2)])).*repmat(sqrt(wts),[3 1]);
    c = xw*xw'/sum(wts);
    
    adj = (sqrt(m)+0.5/gridsize).^2-(sqrt(m)-0.5/gridsize).^2;
    c = c + diag(adj.^2);
    cv(:,:,i) = c;
  end;
end;
fprintf('\n');
